LIBRARY SpinPlugIn;

{ This is a sample Delphi project that demonstrates how to create a simple Spin
  Editor plugin for NeoBook v4/5. Please refer to the "NeoBook Plug-In DOC.rtf"
  file for additional information. }

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  Spin, Interfaces,
  PlugInUtils in 'PlugInUtils.pas',
  Edit1 in 'Edit1.pas' {EditForm1},
  About in 'About.pas' {AboutBoxForm};

{$E nbp}

{$R SPIN.RES}   { <--- Edit this file with a resource editor to change this plug-in's
                       icon which appears in NeoBook's Action list... }


{ NOTE: NeoBook Interface Functions moved to PlugInUtils unit so they can be
  accessed from Edit1.pas. }


{******************* Your Custom Plug-In Functions Go Here ********************}

{ SpinList is used to keep track of the spin controls we create. We can use this
  list to find the controls later if we want to modify them... }
CONST SpinList : TList = NIL;

{ We're going to create our own version of Delphi's TSpinEdit control so we can
  add some NeoBook specific functions to it...}
TYPE TNeoSpinEdit = CLASS( TSpinEdit )
     PRIVATE
       { Private declarations }
       PROCEDURE WMNotifyPlugIn( VAR Msg : TMessage ); MESSAGE WM_NOTIFYPLUGINOBJECT;
     PROTECTED
       { Protected declarations }
       PROCEDURE Change; OVERRIDE;
     PUBLIC
       { Public declarations }
       VariableName,
       RectangleName : STRING;
     PUBLISHED
       { Published declarations }
     END;

{ This procedure is called every time the value of the spin edit control changes.
  We can use this opportunity to update our NeoBook variable... }
PROCEDURE TNeoSpinEdit.Change;
BEGIN
  INHERITED CHANGE;
  IF VariableName > '' THEN
    nbSetVar( PChar( VariableName ), PChar( IntToStr( Value ) ) );
END;

{ This procesure intercepts messages sent by NeoBook... }
PROCEDURE TNeoSpinEdit.WMNotifyPlugIn( VAR Msg : TMessage );
VAR R : TRect;
BEGIN
  CASE Msg.lParam OF
    1 : ; { Page Leave }
    2 : ; { Page Enter }
    3 : ; { Leaving Run mode }
    4 : BEGIN
          { Host rectangle object was resized, so adjust spin control to match... }   
          Windows.GetClientRect( ParentWindow, R );
          SetWindowPos( Handle, 0, 0, 0, R.Right-R.Left, R.Bottom-R.Top, SWP_SHOWWINDOW );
        END;
    5 : IF Msg.wParam <> 0 THEN { Paint control onto DC - used by PrintPage command }
          PaintTo( Msg.wParam, 0, 0 );
    6 : Msg.Result := 0; { Tab Stop - yes we want one }
    7 : Enabled := FALSE; { Host Rectangle Disabled }
    8 : Enabled := TRUE; { Host Rectangle Enabled }
  END;
END;

{ Checks to see if author is using NeoBook version 4.0.9 or higher... }
FUNCTION CheckVer : BOOLEAN;
VAR P : PCHAR;
BEGIN
  Result := FALSE;
  IF Assigned( nbInterface ) THEN
    TRY
      P := NIL;
      nbInterface( 6, P );
      { If NeoBook version of 4.0.9 or higher then P will contain a number.
        If P = NIL then this is an earlier version of NeoBook... }
      Result := P <> NIL
    FINALLY
      FreeStr( P );
    END;
END;

FUNCTION FindSpinEditControl( RectangleName : STRING ) : INTEGER;
VAR I : INTEGER;
BEGIN
  { Search the SpinList to see if there's one that's attached to a NeoBook
    rectangle with this name... }
  Result := -1;
  I     := 0;
  IF Assigned( SpinList ) THEN
    WHILE (I < SpinList.Count) AND (Result = -1) DO
      BEGIN
        IF CompareText( TNeoSpinEdit( SpinList.Items[I] ).RectangleName, RectangleName ) = 0 THEN
          Result := I;
        Inc( I );
      END;
END;

FUNCTION RemoveSpinEditControl( RectangleName : STRING; ShowErrors : BOOLEAN ) : BOOLEAN;
VAR Found : INTEGER;
BEGIN
  Result := FALSE;
  TRY
    { Search the SpinList to see if there's one that's attached to a NeoBook
      rectangle with this name... }
    Found := FindSpinEditControl( RectangleName );

    IF Found > -1 THEN
      BEGIN
        { We found it, so clear the variable and delete the spin edit control... }
        IF TNeoSpinEdit( SpinList.Items[Found] ).VariableName > '' THEN
          nbSetVar( PChar( TNeoSpinEdit( SpinList.Items[Found] ).VariableName ), '' );
        TNeoSpinEdit( SpinList.Items[Found] ).Free;
        { Remove the control from our list too... }
        SpinList.Delete( Found );
        Result := TRUE;
      END
    ELSE IF ShowErrors THEN
      nbSetVar( '[LastError]', PChar( 'There is no spin edit control attached to an object named '+RectangleName+'.' ) );

  EXCEPT ON E : Exception DO
    nbSetVar( '[LastError]', PChar( E.Message ) );
  END;
END;

FUNCTION SetSpinEditControl( RectangleName, NewValue : STRING ) : BOOLEAN;
VAR Found : INTEGER;
BEGIN
  Result := FALSE;
  TRY
    { Search the SpinList to see if there's one that's attached to a NeoBook
      rectangle with this name... }
    Found := FindSpinEditControl( RectangleName );

    IF Found > -1 THEN
      BEGIN
        { We found it, so change the spin edit control's value... }
        TNeoSpinEdit( SpinList.Items[Found] ).Value := StrToInt( NewValue );
        Result := TRUE;
      END
    ELSE nbSetVar( '[LastError]',
      PChar( 'There is no spin edit control attached to an object named '+RectangleName+'.' ) );

  EXCEPT ON E : Exception DO
    nbSetVar( '[LastError]', PChar( E.Message ) );
  END;
END;

FUNCTION CreateSpinEditControl( RectangleName, MinValue, MaxValue, InitialValue, VariableName : STRING ) : BOOLEAN;
VAR Spinner   : TNeoSpinEdit;
    P         : PCHAR;
    ObjHandle : HWND;
    R         : TRect;
BEGIN
  Result := FALSE;
  TRY
    { Check NeoBook version number to make sure author is using NeoBook 4.0.9 or higher... }
    IF CheckVer THEN
      BEGIN

        { We only want one spin edit control per rectangle, so make sure this
          rectangle doesn't already have one... }
        RemoveSpinEditControl( RectangleName, FALSE );

        { Get the window handle of the host NeoBook rectangle object... }
        TRY
          P := NIL;
          SetStr( P, RectangleName );
          nbInterface( 7, P );
          ObjHandle := HWND( StrToInt( P ) );
        FINALLY
          FreeStr( P );
        END;

        IF ObjHandle <> 0 THEN
          BEGIN
            { We found the rectangle's window handle, so attach a spin control to it... }

            Spinner               := TNeoSpinEdit.Create( NIL );

            Spinner.ParentWindow  := ObjHandle;

            Spinner.VariableName  := VariableName;
            Spinner.RectangleName := RectangleName;

            Spinner.MinValue      := StrToInt( MinValue );
            Spinner.MaxValue      := StrToInt( MaxValue );
            Spinner.Value         := StrToInt( InitialValue );

            Spinner.Change;

            Spinner.Visible       := TRUE;

            { Stretch the spinner to match the bounds of the rectangle... }
            Windows.GetClientRect( ObjHandle, R );
            SetWindowPos( Spinner.Handle, 0, 0, 0,
              R.Right-R.Left, R.Bottom-R.Top, SWP_SHOWWINDOW );

            { Add this spin edit control to our list... }
            IF NOT Assigned( SpinList ) THEN
              SpinList := TList.Create;
            SpinList.Add( TObject( Spinner ) );

            Result := TRUE;
          END
        ELSE nbSetVar( '[LastError]', PChar( 'An object named '+RectangleName+' does not exist.' ) );

      END
    ELSE nbSetVar( '[LastError]', 'This function requires NeoBook 4.0.9 or higher.' );

  EXCEPT ON E : Exception DO
    nbSetVar( '[LastError]', PChar( E.Message ) );
  END;
END;


{********** NeoBook Plug-In Functions that Must be Customized by You **********}

{ nbEditAction - called by NeoBook to edit/define on of the Plug-In's commands.
  Plug-In may display a dialog box with fields for user to fill in. Return
  TRUE if successful, FALSE if not.. }
FUNCTION nbEditAction( IDNum      : INTEGER;
                       VAR Params : ARRAY OF PChar ) : BOOLEAN;
BEGIN
  Result := FALSE;
  { Examine the Action string to determine which Plug-In command to execute... }
  CASE IDNum OF
    1 : BEGIN
          { Display our edit form... }
          EditForm1 := TEditForm1.CreateParented( GetActiveWindow );
          TRY
            EditForm1.Caption := 'SpinEditCreate Properties';

            { Fill out the form with the data from NeoBook's parameters... }
            EditForm1.Edit1.Text := Params[0];
            { Try..Except statements prevent bad parameters from causing exception... }
            TRY
              EditForm1.SpinEdit1.Value := StrToInt( Params[1] );
            EXCEPT
            END;
            TRY
              EditForm1.SpinEdit2.Value := StrToInt( Params[2] );
            EXCEPT
            END;
            TRY
              EditForm1.SpinEdit3.Value := StrToInt( Params[3] );
            EXCEPT
            END;
            EditForm1.Edit2.Text := Params[4];

            IF EditForm1.ShowModal = mrOK THEN
              BEGIN
                { The user clicked OK, so transfer the data from the form to our
                  NeoBook parameters... }
                SetStr( Params[0], EditForm1.Edit1.Text );
                SetStr( Params[1], IntToStr( EditForm1.SpinEdit1.Value ) );
                SetStr( Params[2], IntToStr( EditForm1.SpinEdit2.Value ) );
                SetStr( Params[3], IntToStr( EditForm1.SpinEdit3.Value ) );
                SetStr( Params[4], EditForm1.Edit2.Text );

                Result := TRUE;
              END;
            FINALLY
              EditForm1.Free;
            END;
        END;
    2 : BEGIN
          { Display our edit form. Because I'm lazy we're going to use the same
            form as we did for the first action and just hide the controls we
            don't need... }
          EditForm1 := TEditForm1.CreateParented( GetActiveWindow );
          TRY
            EditForm1.Caption := 'SpinEditDelete Properties';

            { Fill out the form with the data from NeoBook's parameters... }
            EditForm1.Edit1.Text := Params[0];

            { Hide the controls we don't need... }
            EditForm1.SpinEdit1.Hide;
            EditForm1.SpinEdit2.Hide;
            EditForm1.SpinEdit3.Hide;
            EditForm1.Edit2.Hide;
            EditForm1.Label3.Hide;
            EditForm1.Label4.Hide;
            EditForm1.Label5.Hide;
            EditForm1.Label6.Hide;

            IF EditForm1.ShowModal = mrOK THEN
              BEGIN
                { The user clicked OK, so transfer the data from the form to our
                  NeoBook parameters... }
                SetStr( Params[0], EditForm1.Edit1.Text );

                Result := TRUE;
              END;
            FINALLY
              EditForm1.Free;
            END;
        END;
    3 : BEGIN
          { Display our edit form. Because I'm lazy we're going to use the same
            form as we did for the first action and just hide the controls we
            don't need... }
          EditForm1 := TEditForm1.CreateParented( GetActiveWindow );
          TRY
            EditForm1.Caption := 'SpinEditSetValue Properties';

            { Fill out the form with the data from NeoBook's parameters... }
            EditForm1.Edit1.Text := Params[0];
            TRY
              EditForm1.SpinEdit3.Value := StrToInt( Params[1] );
            EXCEPT
            END;

            { Hide the controls we don't need... }
            EditForm1.SpinEdit1.Hide;
            EditForm1.SpinEdit2.Hide;
            EditForm1.Edit2.Hide;
            EditForm1.Label3.Hide;
            EditForm1.Label4.Hide;
            EditForm1.Label6.Hide;

            IF EditForm1.ShowModal = mrOK THEN
              BEGIN
                { The user clicked OK, so transfer the data from the form to our
                  NeoBook parameters... }
                SetStr( Params[0], EditForm1.Edit1.Text );
                SetStr( Params[1], IntToStr( EditForm1.SpinEdit3.Value ) );

                Result := TRUE;
              END;
            FINALLY
              EditForm1.Free;
            END;
        END;
  END;
END;


{ nbExecAction - called by NeoBook to execute one of the Plug-In's commands... }
FUNCTION nbExecAction( IDNum      : INTEGER;
                       VAR Params : ARRAY OF PChar ) : BOOLEAN;
BEGIN
  { Examine the Action string to determine which Plug-In command to execute... }
  CASE IDNum OF
    1 : Result := CreateSpinEditControl( Params[0], Params[1], Params[2], Params[3], Params[4] );
    2 : Result := RemoveSpinEditControl( Params[0], TRUE );
    3 : Result := SetSpinEditControl( Params[0], Params[1] );
    ELSE Result := FALSE;
  END;
END;


{ nbMessage - sent by NeoBook to inform plug-in of important NeoBook activities... }
PROCEDURE nbMessage( MsgCode, Reserved : INTEGER );
VAR I : INTEGER;
BEGIN
  { Not all types of plug-ins will care about these messages,
    so they can be ignored if not needed. This procedure must be present even if
    none of the messages are used.

    Possible MsgCode values are:

      1 = Pub has entered run mode
      2 = Pub is about to exit run mode and return to design mode.
      3 = Pub window has been deactivated
      4 = Pub window has been activated
      5 = Pub window has been moved or sized
      6 = Pub is about to display another page
      7 = Pub window has been minimized
      8 = Pub window has been restored

    Reserved value is not currently used
  }

  CASE MsgCode OF
    1 : ; { Don't care }
    2 : IF Assigned( SpinList ) THEN
          BEGIN
            { We're leaving run mode, so delete the SpinList... }
            FOR I := 0 TO SpinList.Count-1 DO
              TNeoSpinEdit( SpinList.Items[I] ).Free;
            SpinList.Free;
            SpinList := NIL;
          END;
    3 : ; { Don't care }
    4 : ; { Don't care }
    5 : ; { Don't care }
    6 : ; { Don't care }
    7 : ; { Don't care }
    8 : ; { Don't care }
  END;

END;


{ nbAbout - called by NeoBook when user wants more information about your plug-in... }
PROCEDURE nbAbout;
BEGIN
  AboutBoxForm := TAboutBoxForm.CreateParented( GetActiveWindow );
  TRY
    AboutBoxForm.ShowModal;
  FINALLY
    AboutBoxForm.Free;
  END;
END;


{ nbInitPlugIn - called by NeoBook to request information about the Plug-In... }
PROCEDURE nbInitPlugIn( WinHandle : HWND; VAR PlugInTitle, PlugInPublisher, PlugInHint : PChar );
BEGIN
  { Save handle of Parent NeoBook App or compiled pub Window - may be required by some Windows functions }
  nbWinHandle := WinHandle;

  { Title of this Plug-In (appears as heading in NeoBook's action list) }
  SetStr( PlugInTitle, 'SpinEdit' );

  { Publisher of this Plug-In }
  SetStr( PlugInPublisher, 'NeoSoft Corp.' );

  { Description of this Plug-In }
  SetStr( PlugInHint, 'Use this plug-in to add a simple spin edit control to NeoBook.' );
END;


{ nbRegisterScriptProcessor - called by NeoBook when registering your plug-in
  Provides access to NeoBook's Action Script Player via nbPlayAction... }
PROCEDURE nbRegisterScriptProcessor( PlayActionProc : POINTER );
BEGIN
  {***************************** DO NOT MODIFY ********************************}
  {*} nbPlayAction := @TPlayActionProc( PlayActionProc );
  {****************************************************************************}
END;

{ nbRegisterInterfaceAccess - called by NeoBook when registering your plug-in
  Provides access to some of NeoBook's design-time interface via nbInterface... }
PROCEDURE nbRegisterInterfaceAccess( InterfaceProc : POINTER );
BEGIN
  {***************************** DO NOT MODIFY ********************************}
  {*} nbInterface := @TInterfaceProc( InterfaceProc );
  {****************************************************************************}
END;

{ nbRegisterStreamAccess - called by NeoBook when registering your plug-in
  Provides access to NeoBook's internal file system... }
PROCEDURE nbRegisterStreamAccess( StreamProc : POINTER );
BEGIN
  {***************************** DO NOT MODIFY ********************************}
  {*} nbFileToStream := @TStreamProc( StreamProc );
  {****************************************************************************}
END;

{ nbRegisterPlugIn - called by NeoBook when it wants you to register your plug-in's actions... }
PROCEDURE nbRegisterPlugIn( AddActionProc, AddFileProc, VarGetFunc, VarSetFunc : POINTER );
BEGIN

  {***************************** DO NOT MODIFY ********************************}
  {*} nbGetVar    := @TVarGetProc( VarGetFunc );
  {*} nbSetVar    := @TVarSetProc( VarSetFunc );
  {*} nbAddAction := @TAddActionProc( AddActionProc );
  {*} nbAddFile   := @TAddFileProc( AddFileProc );
  {****************************************************************************}

  { Call the AddAction procedure for each of Plug-In command.

    Parameters for the ndAddAction procedure:

    Item 1 = Action ID number - must use a unique identifier for each of your actions.
    Item 2 = Action Name
    Item 3 = Action Description
    Item 4 = Array describing each of the Action's parameters - choose from the following:

      ACTIONPARAM_NONE     = Use if action contains no parameters.
      ACTIONPARAM_ALPHA    = Parameter is a string. May contain alpha, numeric, punctuation, etc.
      ACTIONPARAM_ALPHASP  = Parameter is a string that can be spell checked.
      ACTIONPARAM_NUMERIC  = Parameter is a number.
      ACTIONPARAM_MIXED    = May be either numeric or alpha. May contain math expression.
      ACTIONPARAM_FILENAME = Parameter is a file name.
      ACTIONPARAM_VARIABLE = Parameter is a variable name.
      ACTIONPARAM_DATAFILE = Parameter is data file - if not a variable then should be localized.
      ACTIONPARAM_OBJNAME  = Parameter is an object name
      ACTIONPARAM_MENUID   = Parameter is a mennu item id
      ACTIONPARAM_FILENOEX = Parameter is a file name. For with files that you don't want
                             NeoBook to extract automatically. Since the files won't be
                             extracted, you will need to use ndFileToStream.

    Item 5 = Number of parameters required by this action
  }

  {******************* Enter your Plug-In's actions below *********************}


  nbAddAction( 1, 'SpinEditCreate', 'Attach a spin edit control to an existing NeoBook Rectangle object.',
    [ACTIONPARAM_ALPHA,ACTIONPARAM_NUMERIC,ACTIONPARAM_NUMERIC,ACTIONPARAM_NUMERIC,ACTIONPARAM_VARIABLE], 5 );
  nbAddAction( 2, 'SpinEditDelete', 'Remove an spin edit control previously attached to a Rectangle object.',
    [ACTIONPARAM_ALPHA], 1 );
  nbAddAction( 3, 'SpinEditSetValue', 'Change the value of an existing spin edit control.',
    [ACTIONPARAM_ALPHA,ACTIONPARAM_NUMERIC], 2 );


  { Next, if necessary, tell NeoBook what extra files are required for your plug-in.
    These are the files that NeoBook will collect when compiling publications that
    use this plug-in. If your plug-in uses any data files, drivers or special DLLs
    this is where you will tell NeoBook about it. It is NOT necessary to include
    the name of the plug-in itself since NeoBook will automatically assume that
    it is required.

    Parameters for the ndAddFile procedure:

    Item 1 = File name including correct drive and path
    Item 2 = TRUE to add the file, FALSE to remove the file

    For example:

    nbAddFile( 'c:\path\somefile.xyz', TRUE );
  }


END;


PROCEDURE DLLHandler( Reason : INTEGER );
BEGIN
  IF Reason = DLL_PROCESS_DETACH THEN
    BEGIN
      { If this plug-in requires any special processing before being unloaded from
        memory, do that here. Leave blank if no special processing is needed. }

    END;
END;


{***** Export the required functions for NeoBook interface. Do NOT modify *****}
{*}EXPORTS nbEditAction;
{*}EXPORTS nbExecAction;
{*}EXPORTS nbInitPlugIn;
{*}EXPORTS nbRegisterPlugIn;
{*}EXPORTS nbRegisterScriptProcessor;
{*}EXPORTS nbRegisterInterfaceAccess;
{*}EXPORTS nbRegisterStreamAccess;
{*}EXPORTS nbMessage;
{*}EXPORTS nbAbout;
{******************************************************************************}

BEGIN
//  DLLProc := @DLLHandler;
END.






